; Name:    TextView 1.0
; Author:  Kouri (Kouri@hotbot.com)
; Date:    April 22, 1999
; Info:    Allows you to view ASCII text files created by Txt282p.exe.
;          See Txt282p.c for more information.
; Usage:   Run TextView from CrASH. A list of all the valid text files on
;          your calc will be shown. If you can't run TextView, then you
;          don't have any valid text files. Move through the list with the
;          arrow keys. [ENTER] selects a file to view. [CLEAR] quits. Once
;          your are viewing a file, use the up and down arrows to browse
;          through the file. [DEL] brings you to the start of the file, and
;          [STAT] brings you to the end. [CLEAR] returns to the Open File
;          Menu. I used CR_KHAND so you can also change the contrast and
;          turn off the calc. Turning off the calc while viewing a file will
;          not crash it, but your graph screen won't display correctly
;          unless you enter and then exit CrASH again.
; Credits: Jimmy Mrdell and Florent Dhordain for SQRXZ 82, and Andreas Ess
;          and Sam Heald for Plain Jump 82, whose programs showed me how to
;          search the VAT. Most of this program is just modifications of
;          the routines used in theirs. THANKS GUYS!

#include CrASH82.inc
.db "TextView 1.0 by Kouri",0

dataptr     = TEXT_MEM		; A pointer to the current string
num_files   = TEXT_MEM+2	; The number of text files on the calc
top         = TEXT_MEM+3	; A pointer offset used by Open File Menu
num_choices = TEXT_MEM+4	; The number of choices in the file list
cur_choice  = TEXT_MEM+5	; The current choice in the list
files       = TEXT_MEM+6	; See comment below

; TEXT_MEM is used as a storage area for program addresses because CrASH
; automatically clears it before starting any program. This allows up to 61
; program addresses to be stored ((128-6)/2 == 61). I could have allowed
; for more, but that would require using APD_BUF, which then requires
; interrupt checking and a routine to clear the APD_BUF everytime TxtView
; runs. I think 61 is plenty of programs anyway.

 ld hl,VAT_START
 ld ix,files

SearchLoop:
 ld de,($8D12)			; End of progtable - 1
 inc de
 call CP_HL_DE \ jr c,SearchDone
 ld a,(hl)			; Save type
 dec hl				; Get past type byte
 sub 5 \ jr z,FoundProgram	; Type 5 == regular program
 dec a \ jr z,FoundProgram	; Type 6 == edit-locked program
 ld b,5				; Not a program, skip next 5 bytes

SkipName:
 dec hl
 djnz SkipName
 jr SearchLoop

FoundProgram:
 ld e,(hl)
 dec hl
 ld d,(hl)			; de -> program data
 push hl			; Save pointer
 ex de,hl			; hl -> program data
 inc hl				; Skip program length
 inc hl
 ld a,(hl)			; Get first byte
 inc a \ jr nz,KeepSearching	; Is first byte $FF?
 inc hl				; Go to next byte
 ld a,(hl)
 or a \ jr nz,KeepSearching	; Is second byte 0?
 inc hl				; This one's a keeper!
 ld (ix),l			; Store address of program data to
 inc ix				; memory in LSB order
 ld (ix),h
 inc ix				; Point to address for next entry
 ld hl,num_files
 inc (hl)			; Increase number of files found

KeepSearching:
 pop hl				; Restore pointer
 dec hl				; Point to namelength byte
 ld b,(hl)			; Save namelength
 inc b				; Inc so we skip that byte too
 jr SkipName

SearchDone:
 ld a,(num_files)
 or a \ ret z			; No files found, return to shell

OpenFileMenu:
 res 7,(iy+$14)			; Do not write text to GRAPH_MEM
 ROM_CALL(CLEARLCD)
 ld de,$0001
 ld (GRAF_CURS),de
 ld hl,Title
 ROM_CALL(D_ZM_STR)		; Display Title
 ld d,57
 ld (GRAF_CURS),de		; Load GRAF_CURS with (57,1)
 ROM_CALL(D_ZM_STR)		; Display Instructions
 ld a,-7
 call MI_NoRowCalc		; Invert the row
 ld a,50
 call MI_NoRowCalc		; Invert the row
 ld hl,$0701
 ld (GRAF_CURS),hl		; Go to line after the title
 ld hl,files
 ld a,(top)
 add a,a
 ld d,0
 ld e,a				; de = 2*top
 add hl,de			; hl -> pointer to files
 ld b,8				; 8 files max to be shown

DisplayFile:
 ld e,(hl)
 inc hl
 ld d,(hl)			; de -> program data
 inc hl				; Point to next address
 xor a
 or d \ jr z,EndOfList		; If d == 0, end of list reached
 push hl			; Save pointer
 ex de,hl			; hl -> program data
 ROM_CALL(D_ZM_STR)		; Show file description
 ld hl,GRAF_CURS
 ld (hl),1			; Set X coord to 1
 inc hl
 ld a,(hl)
 add a,6
 ld (hl),a			; Increase Y coord by 6
 pop hl				; Restore pointer
 djnz DisplayFile

EndOfList:
 ld a,8
 sub b				; a = number of files in current list
 ld (num_choices),a
 xor a

UpdateChoice:
 ld (cur_choice),a
 call MarkItem			; Mark the current selection

Choose:
 call CR_KHAND
 cp K_CLEAR \ ret z
 cp K_ENTER \ jr z,SelectFile
 cp K_RIGHT \ jr z,NextPage
 cp K_DOWN  \ jr z,CursorDown
 cp K_UP    \ jr nz,Choose

CursorUp:
 ld a,(cur_choice)
 call MarkItem			; Unmark the current selected file
 or a \ jr nz,NoWrap		; Check if at top of list
 ld a,(num_choices)		; a = bottom of list + 1

NoWrap:
 dec a
 jr UpdateChoice

CursorDown:
 ld a,(cur_choice)
 call MarkItem			; Unmark the current selected file
 inc a
 ld hl,num_choices
 cp (hl) \ jr nz,UpdateChoice	; Check if bottom of list passed
 xor a				; If so, current file is 0
 jr UpdateChoice

NextPage:
 ld a,(top)
 add a,8			; Add top of list with 8
 ld hl,num_files
 cp (hl) \ jr c,ChangePage	; Check if we can go to next page
 xor a				; If not, start at page 0

ChangePage:
 ld (top),a
 jp OpenFileMenu

SelectFile:
 ld a,(top)
 ld hl,cur_choice
 add a,(hl)
 ld hl,files
 add a,a
 ld d,0
 ld e,a
 add hl,de			; hl -> pointer to current file
 ld e,(hl)
 inc hl
 ld d,(hl)			; de = file to open
 ex de,hl
 set 7,(iy+$14)			; Write text to GRAPH_MEM

UpdateDisplay:
 ld de,$0100
 ld (dataptr),hl		; Save the pointer

DisplayLoop:
 ld (GRAF_CURS),de
 ROM_CALL(D_ZM_STR)
 ld a,$FF
 cp (hl) \ jr z,DisplayScreen
 ld a,d \ add a,6 \ ld d,a
 cp 61 \ jr nz,DisplayLoop

DisplayScreen:
 call CR_GRBCopy		; Display the screen
 ld hl,GRAPH_MEM
 ld de,GRAPH_MEM+1
 ld (hl),0
 ld bc,767
 ldir				; And then clear the GRAPH_MEM

Main:
 call CR_KHAND
 ld bc,$FFFF			; For cpdr and cpir
 ld hl,(dataptr)
 cp K_DOWN  \ jr z,Down
 cp K_DEL   \ jr z,Home
 cp K_STAT  \ jr z,End
 cp K_CLEAR \ jp z,OpenFileMenu
 cp K_UP    \ jr nz,Main

Up:
 dec hl \ dec hl		; Get past the intial zero
 ld a,$FF \ cp (hl) \ jr z,Main	; Check for beginning of file
 xor a \ cpdr			; Go to previous zero
_Up:
 inc hl \ inc hl		; Point to the immediate string
 jr UpdateDisplay

Down:
 xor a \ cpir			; Go to next zero
 dec a \ cp (hl) \ jr z,Main	; Check for EOF
 jr UpdateDisplay

Home:
 ld a,$FF \ cpdr
 inc hl
 jr _Up

End:
 ld a,$FF \ cpir
 dec hl
 jr Up

MarkItem:
 ld h,a				; save the row #
 add a,a
 add a,a			; 4*a
 add a,h
 add a,h			; 6*a

MI_NoRowCalc:
 ld e,a
 ld a,7				; 7 lines to invert
 ld b,a				; Go right after each write
 call LCDOut

MI_NextRow:
 ld a,e
 add a,b
 add a,$80+7-1			; Go to line a+b-1
 call LCDOut
 ld c,$20			; Invert one entire row

InvertRow:
 ld a,c				; Go to column...
 call LCDOut
 call DISP_DELAY
 in a,(P_LCD_DATA)		; Dummy read
 call DISP_DELAY
 in a,(P_LCD_DATA)
 cpl
 ld d,a
 ld a,c
 call LCDOut
 ld a,d
 call DISP_DELAY
 out (P_LCD_DATA),a		; and show the inverted pixel octet
 inc c
 ld a,c
 cp $2C				; Row finished?
 jr nz,InvertRow
 djnz MI_NextRow
 ld a,h
 ret

LCDOut:
 call DISP_DELAY
 out (P_LCD_CON),a
 ret

Title:
.db "TextView - Select file",0

Instructions:
.db 193,30," ] ",193,31," ] ",193,28,"] ",193,"Enter] ",193,"Clear]",0
